source_term = "Source term"
initial_value = "Initial value"
initial_derivative = "Initial Derivative"
time_interval = "Time interval"
samples = "Samples"

source_term_symbol = "f(t, x, y=x')"
initial_value_symbol = "x(0)"
initial_derivative_symbol = "x'(0)"
time_interval_symbol = "T"
samples_symbol = "N"
