left_boundary_values = "Left boundary values"
right_boundary_values = "Right boundary values"
initial_values = "Initial Values"
source_term = "Source term (optional)"
diffusivity = "Diffusivity"
length = "Length"
time_interval = "Time interval"
samples = "Samples"

left_boundary_type_symbol = "φ₁(t)"
right_boundary_type_symbol = "φ₂(t)"
initial_values_symbol = "u(0, x)"
source_term_symbol = "s(t, x)"
heat_equation_initial_value_symbol = "u(0, x)"
heat_equation_source_term_symbol = "s(t, x)"
diffusivity_symbol = "α"
length_symbol = "L"
time_interval_symbol = "T"
samples_symbol = "N"
